@@ -5,21 +5,20 @@
  * Copyright (C) 2001-2003, Ximian, Inc.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
+#include <ctype.h>
 
 #include "soup-headers.h"
-#include "soup.h"
+#include "soup-misc.h"
+#include "soup-uri.h"
 
 /**
  * soup_headers_parse:
  * @str: the header string (including the Request-Line or Status-Line,
- *   but not the trailing blank line)
- * @len: length of @str
+ * and the trailing blank line)
+ * @len: length of @str up to (but not including) the terminating blank line.
  * @dest: #SoupMessageHeaders to store the header values in
  *
  * Parses the headers of an HTTP request or response in @str and
@@ -38,13 +37,19 @@ soup_headers_parse (const char *str, int len, SoupMessageHeaders *dest)
 {
 	const char *headers_start;
 	char *headers_copy, *name, *name_end, *value, *value_end;
-	char *eol, *sol, *p;
-	gsize copy_len;
+	char *eol, *sol;
 	gboolean success = FALSE;
 
 	g_return_val_if_fail (str != NULL, FALSE);
 	g_return_val_if_fail (dest != NULL, FALSE);
 
+	/* Technically, the grammar does allow NUL bytes in the
+	 * headers, but this is probably a bug, and if it's not, we
+	 * can't deal with them anyway.
+	 */
+	if (memchr (str, '\0', len))
+		return FALSE;
+
 	/* As per RFC 2616 section 19.3, we treat '\n' as the
 	 * line terminator, and '\r', if it appears, merely as
 	 * ignorable trailing whitespace.
@@ -54,41 +59,22 @@ soup_headers_parse (const char *str, int len, SoupMessageHeaders *dest)
 	headers_start = memchr (str, '\n', len);
 	if (!headers_start)
 		return FALSE;
-	/* No '\0's in the Request-Line / Status-Line */
-	if (memchr (str, '\0', headers_start - str))
-		return FALSE;
 
 	/* We work on a copy of the headers, which we can write '\0's
 	 * into, so that we don't have to individually g_strndup and
 	 * then g_free each header name and value.
 	 */
-	copy_len = len - (headers_start - str);
-	headers_copy = g_malloc (copy_len + 1);
-	memcpy (headers_copy, headers_start, copy_len);
-	headers_copy[copy_len] = '\0';
+	headers_copy = g_strndup (headers_start, len - (headers_start - str));
 	value_end = headers_copy;
 
-	/* There shouldn't be any '\0's in the headers already, but
-	 * this is the web we're talking about.
-	 */
-	while ((p = memchr (headers_copy, '\0', copy_len))) {
-		memmove (p, p + 1, copy_len - (p - headers_copy));
-		copy_len--;
-	}
-
 	while (*(value_end + 1)) {
 		name = value_end + 1;
 		name_end = strchr (name, ':');
-
-		/* Reject if there is no ':', or the header name is
-		 * empty, or it contains whitespace.
-		 */
-		if (!name_end ||
-		    name_end == name ||
-		    name + strcspn (name, " \t\r\n") < name_end) {
-			/* Ignore this line. Note that if it has
-			 * continuation lines, we'll end up ignoring
-			 * them too since they'll start with spaces.
+		if (!name_end || name + strcspn (name, " \t\r\n") < name_end) {
+			/* Bad header; just ignore this line. Note
+			 * that if it has continuation lines, we'll
+			 * end up ignoring them too since they'll
+			 * start with spaces.
 			 */
 			value_end = strchr (name, '\n');
 			if (!value_end)
@@ -141,10 +127,6 @@ soup_headers_parse (const char *str, int len, SoupMessageHeaders *dest)
 			eol--;
 		*eol = '\0';
 
-		/* convert (illegal) '\r's to spaces */
-		for (p = strchr (value, '\r'); p; p = strchr (p, '\r'))
-			*p = ' ';
-
 		soup_message_headers_append (dest, name, value);
         }
 	success = TRUE;
@@ -156,8 +138,8 @@ done:
 
 /**
  * soup_headers_parse_request:
- * @str: the headers (up to, but not including, the trailing blank line)
- * @len: length of @str
+ * @str: the header string (including the trailing blank line)
+ * @len: length of @str up to (but not including) the terminating blank line.
  * @req_headers: #SoupMessageHeaders to store the header values in
  * @req_method: (out) (allow-none): if non-%NULL, will be filled in with the
  * request method
@@ -187,7 +169,7 @@ soup_headers_parse_request (const char          *str,
 	unsigned long major_version, minor_version;
 	char *p;
 
-	g_return_val_if_fail (str != NULL, SOUP_STATUS_MALFORMED);
+	g_return_val_if_fail (str && *str, SOUP_STATUS_MALFORMED);
 
 	/* RFC 2616 4.1 "servers SHOULD ignore any empty line(s)
 	 * received where a Request-Line is expected."
@@ -323,7 +305,7 @@ soup_headers_parse_status_line (const char       *status_line,
 	if (code_end != code_start + 3)
 		return FALSE;
 	code = atoi (code_start);
-	if (code < 100 || code > 999)
+	if (code < 100 || code > 599)
 		return FALSE;
 	if (status_code)
 		*status_code = code;
@@ -343,8 +325,8 @@ soup_headers_parse_status_line (const char       *status_line,
 
 /**
  * soup_headers_parse_response:
- * @str: the headers (up to, but not including, the trailing blank line)
- * @len: length of @str
+ * @str: the header string (including the trailing blank line)
+ * @len: length of @str up to (but not including) the terminating blank line.
  * @headers: #SoupMessageHeaders to store the header values in
  * @ver: (out) (allow-none): if non-%NULL, will be filled in with the HTTP
  * version
@@ -370,7 +352,7 @@ soup_headers_parse_response (const char          *str,
 {
 	SoupHTTPVersion version;
 
-	g_return_val_if_fail (str != NULL, FALSE);
+	g_return_val_if_fail (str && *str, FALSE);
 
 	/* Workaround for broken servers that send extra line breaks
 	 * after a response, which we then see prepended to the next
@@ -611,7 +593,11 @@ soup_header_parse_quality_list (const char *header, GSList **unacceptable)
 void
 soup_header_free_list (GSList *list)
 {
-	g_slist_free_full (list, g_free);
+	GSList *l;
+
+	for (l = list; l; l = l->next)
+		g_free (l->data);
+	g_slist_free (list);
 }
 
 /**
@@ -630,13 +616,11 @@ gboolean
 soup_header_contains (const char *header, const char *token)
 {
 	const char *end;
-	guint len;
+	guint len = strlen (token);
 
 	g_return_val_if_fail (header != NULL, FALSE);
 	g_return_val_if_fail (token != NULL, FALSE);
 
-	len = strlen (token);
-
 	header = skip_delims (header, ',');
 	while (*header) {
 		end = skip_item (header, ',');
@@ -717,11 +701,14 @@ parse_param_list (const char *header, char delim)
 	char *item, *eq, *name_end, *value;
 	gboolean override;
 
+	list = parse_list (header, delim);
+	if (!list)
+		return NULL;
+
 	params = g_hash_table_new_full (soup_str_case_hash, 
 					soup_str_case_equal,
 					g_free, NULL);
 
-	list = parse_list (header, delim);
 	for (iter = list; iter; iter = iter->next) {
 		item = iter->data;
 		override = FALSE;
