@@ -25,18 +25,21 @@
 #include <config.h>
 #endif
 
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
+
+#define LIBSOUP_USE_UNSTABLE_REQUEST_API
 
 #include "soup-request.h"
-#include "soup.h"
 #include "soup-requester.h"
+#include "soup-session.h"
+#include "soup-uri.h"
 
 /**
  * SECTION:soup-request
  * @short_description: Protocol-independent streaming request interface
  *
- * A #SoupRequest is created by #SoupSession, and represents a request
- * to retrieve a particular URI.
+ * A #SoupRequest is created by #SoupRequester, and represents a
+ * request to retrieve a particular URI.
  */
 
 /**
@@ -44,9 +47,15 @@
  *
  * A request to retrieve a particular URI.
  *
- * Since: 2.42
+ * Since: 2.34
  */
 
+static void soup_request_initable_interface_init (GInitableIface *initable_interface);
+
+G_DEFINE_TYPE_WITH_CODE (SoupRequest, soup_request, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
+						soup_request_initable_interface_init))
+
 enum {
 	PROP_0,
 	PROP_URI,
@@ -58,17 +67,10 @@ struct _SoupRequestPrivate {
 	SoupSession *session;
 };
 
-static void soup_request_initable_interface_init (GInitableIface *initable_interface);
-
-G_DEFINE_TYPE_WITH_CODE (SoupRequest, soup_request, G_TYPE_OBJECT,
-                         G_ADD_PRIVATE (SoupRequest)
-			 G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
-						soup_request_initable_interface_init))
-
 static void
 soup_request_init (SoupRequest *request)
 {
-	request->priv = soup_request_get_instance_private (request);
+	request->priv = G_TYPE_INSTANCE_GET_PRIVATE (request, SOUP_TYPE_REQUEST, SoupRequestPrivate);
 }
 
 static void
@@ -76,8 +78,10 @@ soup_request_finalize (GObject *object)
 {
 	SoupRequest *request = SOUP_REQUEST (object);
 
-	g_clear_pointer (&request->priv->uri, soup_uri_free);
-	g_clear_object (&request->priv->session);
+	if (request->priv->uri)
+		soup_uri_free (request->priv->uri);
+	if (request->priv->session)
+		g_object_unref (request->priv->session);
 
 	G_OBJECT_CLASS (soup_request_parent_class)->finalize (object);
 }
@@ -137,7 +141,7 @@ soup_request_initable_init (GInitable     *initable,
 	gboolean ok;
 
 	if (!request->priv->uri) {
-		g_set_error (error, SOUP_REQUEST_ERROR, SOUP_REQUEST_ERROR_BAD_URI,
+		g_set_error (error, SOUP_REQUESTER_ERROR, SOUP_REQUESTER_ERROR_BAD_URI,
 			     _("No URI provided"));
 		return FALSE;
 	}
@@ -145,10 +149,10 @@ soup_request_initable_init (GInitable     *initable,
 	ok = SOUP_REQUEST_GET_CLASS (initable)->
 		check_uri (request, request->priv->uri, error);
 
-	if (!ok && error && !*error) {
+	if (!ok && error) {
 		char *uri_string = soup_uri_to_string (request->priv->uri, FALSE);
-		g_set_error (error, SOUP_REQUEST_ERROR, SOUP_REQUEST_ERROR_BAD_URI,
-			     _("Invalid “%s” URI: %s"),
+		g_set_error (error, SOUP_REQUESTER_ERROR, SOUP_REQUESTER_ERROR_BAD_URI,
+			     _("Invalid '%s' URI: %s"),
 			     request->priv->uri->scheme,
 			     uri_string);
 		g_free (uri_string);
@@ -172,18 +176,13 @@ soup_request_default_send_async (SoupRequest          *request,
 				 GAsyncReadyCallback   callback,
 				 gpointer              user_data)
 {
-	GTask *task;
-	GInputStream *stream;
-	GError *error = NULL;
+	GSimpleAsyncResult *simple;
 
-	task = g_task_new (request, cancellable, callback, user_data);
-
-	stream = soup_request_send (request, cancellable, &error);
-	if (stream)
-		g_task_return_pointer (task, stream, g_object_unref);
-	else
-		g_task_return_error (task, error);
-	g_object_unref (task);
+	simple = g_simple_async_result_new (G_OBJECT (request),
+					    callback, user_data,
+					    soup_request_default_send_async);
+	g_simple_async_result_complete_in_idle (simple);
+	g_object_unref (simple);
 }
 
 static GInputStream *
@@ -191,7 +190,9 @@ soup_request_default_send_finish (SoupRequest          *request,
 				  GAsyncResult         *result,
 				  GError              **error)
 {
-	return g_task_propagate_pointer (G_TASK (result), error);
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (request), soup_request_default_send_async), NULL);
+
+	return soup_request_send (request, NULL, error);
 }
 
 /**
@@ -203,13 +204,10 @@ soup_request_default_send_finish (SoupRequest          *request,
  * Synchronously requests the URI pointed to by @request, and returns
  * a #GInputStream that can be used to read its contents.
  *
- * Note that you cannot use this method with #SoupRequests attached to
- * a #SoupSessionAsync.
- *
  * Return value: (transfer full): a #GInputStream that can be used to
  *   read from the URI pointed to by @request.
  *
- * Since: 2.42
+ * Since: 2.34
  */
 GInputStream *
 soup_request_send (SoupRequest          *request,
@@ -230,10 +228,7 @@ soup_request_send (SoupRequest          *request,
  * Begins an asynchronously request for the URI pointed to by
  * @request.
  *
- * Note that you cannot use this method with #SoupRequests attached to
- * a #SoupSessionSync.
- *
- * Since: 2.42
+ * Since: 2.34
  */
 void
 soup_request_send_async (SoupRequest         *request,
@@ -256,7 +251,7 @@ soup_request_send_async (SoupRequest         *request,
  * Return value: (transfer full): a #GInputStream that can be used to
  *   read from the URI pointed to by @request.
  *
- * Since: 2.42
+ * Since: 2.34
  */
 GInputStream *
 soup_request_send_finish (SoupRequest          *request,
@@ -272,6 +267,8 @@ soup_request_class_init (SoupRequestClass *request_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (request_class);
 
+	g_type_class_add_private (request_class, sizeof (SoupRequestPrivate));
+
 	request_class->check_uri = soup_request_default_check_uri;
 	request_class->send_async = soup_request_default_send_async;
 	request_class->send_finish = soup_request_default_send_finish;
@@ -280,50 +277,20 @@ soup_request_class_init (SoupRequestClass *request_class)
 	object_class->set_property = soup_request_set_property;
 	object_class->get_property = soup_request_get_property;
 
-	/**
-	 * SOUP_REQUEST_URI:
-	 *
-	 * Alias for the #SoupRequest:uri property, qv.
-	 *
-	 * Since: 2.42
-	 */
-	/**
-	 * SoupRequest:uri:
-	 *
-	 * The request URI.
-	 *
-	 * Since: 2.42
-	 */
 	g_object_class_install_property (
 		 object_class, PROP_URI,
 		 g_param_spec_boxed (SOUP_REQUEST_URI,
 				     "URI",
 				     "The request URI",
 				     SOUP_TYPE_URI,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				     G_PARAM_STATIC_STRINGS));
-	/**
-	 * SOUP_REQUEST_SESSION:
-	 *
-	 * Alias for the #SoupRequest:session property, qv.
-	 *
-	 * Since: 2.42
-	 */
-	/**
-	 * SoupRequest:session:
-	 *
-	 * The request's #SoupSession.
-	 *
-	 * Since: 2.42
-	 */
+				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (
 		 object_class, PROP_SESSION,
 		 g_param_spec_object (SOUP_REQUEST_SESSION,
 				      "Session",
 				      "The request's session",
 				      SOUP_TYPE_SESSION,
-				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				      G_PARAM_STATIC_STRINGS));
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -340,7 +307,7 @@ soup_request_initable_interface_init (GInitableIface *initable_interface)
  *
  * Return value: (transfer none): @request's URI
  *
- * Since: 2.42
+ * Since: 2.34
  */
 SoupURI *
 soup_request_get_uri (SoupRequest *request)
@@ -356,7 +323,7 @@ soup_request_get_uri (SoupRequest *request)
  *
  * Return value: (transfer none): @request's #SoupSession
  *
- * Since: 2.42
+ * Since: 2.34
  */
 SoupSession *
 soup_request_get_session (SoupRequest *request)
@@ -368,14 +335,12 @@ soup_request_get_session (SoupRequest *request)
  * soup_request_get_content_length:
  * @request: a #SoupRequest
  *
- * Gets the length of the data represented by @request. For most
- * request types, this will not be known until after you call
- * soup_request_send() or soup_request_send_finish().
+ * Gets the length of the data represented by @request.
  *
  * Return value: the length of the data represented by @request,
  *   or -1 if not known.
  *
- * Since: 2.42
+ * Since: 2.34
  */
 goffset
 soup_request_get_content_length (SoupRequest *request)
@@ -387,17 +352,12 @@ soup_request_get_content_length (SoupRequest *request)
  * soup_request_get_content_type:
  * @request: a #SoupRequest
  *
- * Gets the type of the data represented by @request. For most request
- * types, this will not be known until after you call
- * soup_request_send() or soup_request_send_finish().
+ * Gets the type of the data represented by @request.
  *
- * As in the HTTP Content-Type header, this may include parameters
- * after the MIME type.
+ * Return value: the type of the data represented by @request,
+ *   or %NULL if not known.
  *
- * Return value: (nullable): the type of the data represented by
- *   @request, or %NULL if not known.
- *
- * Since: 2.42
+ * Since: 2.34
  */
 const char *
 soup_request_get_content_type (SoupRequest  *request)
